package pdv;

import junit.framework.*;

public class PDVTeste extends TestCase
{
	static public Test suite( )
	{
		return new TestSuite( PDVTeste.class );
	}

	static public void main( String[] args )
	{
		//junit.textui.TestRunner.run( suite( ) );
		String[] params = {"pdv.PDVTeste"};
		junit.swingui.TestRunner.main( params );
	}

	public void testeEntrarItem( )
	{
		PDV pdv;
		Venda venda;
		boolean caught;

		/* upc valido e quantidade > 0 */
		pdv = new PDV( new CatalogoProdutos( ) );
		Assert.assertEquals( null, pdv.getVenda( ) );
		pdv.entrarItem( 1001, 2 );
		venda = pdv.getVenda( );
		Assert.assertTrue( !venda.isCompleta( ) );
		Assert.assertEquals( 1, venda.totalItens( ) );
		Assert.assertEquals( (float)( 2 * 10.5 ), venda.total( ), 0.03 );
		pdv.entrarItem( 1002, 4 );
		venda = pdv.getVenda( );
		Assert.assertTrue( !venda.isCompleta( ) );
		Assert.assertEquals( 2, venda.totalItens( ) );
		Assert.assertEquals( (float)( 2 * 10.5 + 4 * 20.35 ), venda.total( ), 0.03 );
		pdv.entrarItem( 1006, 5 );
		venda = pdv.getVenda( );
		Assert.assertTrue( !venda.isCompleta( ) );
		Assert.assertEquals( 3, venda.totalItens( ) );
		Assert.assertEquals( (float)( 2 * 10.5 + 4 * 20.35 + 5 * 6.75 ), venda.total( ), 0.03 );

		/* upc valido e quantidade <= 0 */
		caught = true;

		pdv = new PDV( new CatalogoProdutos( ) );
		try
		{
			pdv.entrarItem( 1004, -2 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar uma excecao quando quantidade < 0." );

		/* upc invalido e quantidade > 0 */
		caught = true;

		pdv = new PDV( new CatalogoProdutos( ) );
		try
		{
			pdv.entrarItem( 12345, 3 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar uma IllegalArgumentException quando upc eh invalido." );

		/* upc invalido e quantidade < 0 */
		caught = true;

		pdv = new PDV( new CatalogoProdutos( ) );
		try
		{
			pdv.entrarItem( 12345, -9 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar uma IllegalArgumentException quando upc eh invalido e quantidade < 0." );
	}

	public void testeEntrarItemNosValoresLimite( )
	{
		PDV pdv;
		boolean caught;

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1004, +1 );
		pdv.entrarItem( 1004, Integer.MAX_VALUE );

		caught = true;
		try
		{
			pdv.entrarItem( 1004, 0 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar IllegalArgumentException quando quantidade <= 0." );

		caught = true;
		try
		{
			pdv.entrarItem( 1004, -1 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar IllegalArgumentException quando quantidade <= 0." );

		caught = true;
		try
		{
			pdv.entrarItem( 1004, Integer.MIN_VALUE );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.entrarItem( ) deveria lancar IllegalArgumentException quando quantidade <= 0." );
	}

	public void testeTerminarVenda( )
	{
		PDV pdv;

		/* Venda criada anteriormente */
		pdv = new PDV( new CatalogoProdutos( ) );
		Assert.assertEquals( null, pdv.getVenda( ) );
		pdv.entrarItem( 1004, 3 );
		Assert.assertTrue( pdv.getVenda( ) != null );
		Assert.assertTrue( !pdv.getVenda( ).isCompleta( ) );
		pdv.terminarVenda( );
		Assert.assertTrue( pdv.getVenda( ).isCompleta( ) );
		/* Venda nao criada anteriormente */
		pdv = new PDV( new CatalogoProdutos( ) );
		Assert.assertEquals( null, pdv.getVenda( ) );
		boolean caught = true;
		try
		{
			pdv.terminarVenda( );
			caught = false;
		}
		catch( IllegalStateException e )
		{
		}
		if( !caught )
			Assert.fail( "PDV.terminarVenda( ) deveria ter lancado uma IllegalStateException quando uma Venda nao foi criada anteriormente." );
	}

	public void testeRegistrarPagamentoComQuantiaIgualATotal( )
	{
		PDV pdv;
		boolean caught;

		/* Venda criada anteriormente, venda completa, venda ainda nao paga e quantia = total */
		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1005, 10 );
		pdv.terminarVenda( );
		pdv.registrarPagamento( (float)( 10 * 5.99 ) );
		/* Venda jah paga */
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 10 * 5.99 ) );
			caught = false;
		}
		catch( IllegalStateException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga mais de uma vez." );
	}
	
	public void testeRegistrarPagamentoComQuantiaMaiorQueTotal( )
	{
		PDV pdv;
		boolean caught;

		/* Venda criada anteriormente, venda completa, venda ainda nao paga e quantia > total */
		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1005, 10 );
		pdv.terminarVenda( );
		pdv.registrarPagamento( (float)( 10 * 5.99 + 10 ) );
		/* Venda jah paga */
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 10 * 5.99 + 10 ) );
			caught = false;
		}
		catch( IllegalStateException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga mais de uma vez." );
	}

	public void testeRegistrarPagamentoComQuantiaMenorQueTotal( )
	{
		PDV pdv;
		boolean caught;

		/* venda criada anteriormente, venda completa, venda ainda nao paga e quantia < total */
		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1005, 10 );
		pdv.terminarVenda( );
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 10 * 5.99 - 4 ) );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga com uma quantia menor do que o total." );
	}

	public void testeRegistrarPagamentoSemCompletarVenda( )
	{
		PDV pdv;
		boolean caught;

		/* venda criada anteriormente, venda nao completa */
		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1005, 10 );
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 10 * 5.99 ) );
			caught = false;
		}
		catch( IllegalStateException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga se nao estiver completa." );
	}

	public void testeRegistrarPagamentoSemCriarVenda( )
	{
		PDV pdv;
		boolean caught;

		/* venda nao criada anteriormente */
		pdv = new PDV( new CatalogoProdutos( ) );
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 1 ) );
			caught = false;
		}
		catch( IllegalStateException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga se nao existir." );	
	}

	public void testeRegistrarPagamentoNosValoresLimite( )
	{
		PDV pdv;
		boolean caught;

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1003, 1 );
		pdv.terminarVenda( );
		pdv.registrarPagamento( pdv.getVenda( ).total( ) );

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1003, 1 );
		pdv.terminarVenda( );
		pdv.registrarPagamento( pdv.getVenda( ).total( ) + 1 );

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1003, 1 );
		pdv.terminarVenda( );
		caught = true;
		try
		{
			pdv.registrarPagamento( pdv.getVenda( ).total( ) - 1 );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga com uma quantia menor do que o total." );

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1003, 1 );
		pdv.terminarVenda( );
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( 0.0 ) );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga com uma quantia menor do que o total." );

		pdv = new PDV( new CatalogoProdutos( ) );
		pdv.entrarItem( 1003, 1 );
		pdv.terminarVenda( );
		caught = true;
		try
		{
			pdv.registrarPagamento( (float)( -1 ) );
			caught = false;
		}
		catch( IllegalArgumentException e )
		{
		}
		if( !caught )
			Assert.fail( "Uma venda nao pode ser paga com uma quantia negativa." );
	}

} /* fim da classe PDVTeste */